﻿using FwsPoly.SDK.Crypts;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.IO.Ports;

namespace FwsPoly.SDK
{
    public class Poly
    {
        public event Action<int>? EventPolyUiUpdate;

        public bool bScanWaveEnable;
        public bool ScanRunning;

        public double StartPoint_nm;
        public double EndPoint_nm;

        const short INDEX_FWHM = 0;
        const short INDEX_F1 = 1;
        const short INDEX_F2 = 2;
        const short INDEX_F3 = 3;
        const short INDEX_F1W = 4;
        const short INDEX_F2W = 5;
        const short INDEX_F4W = 6;
        const int COMMAND_TIME_OUT = 3000;
        const double C107 = 10000000.0;

        bool polyEnabled, polyDllStatus, bcheckOrg,
            bSetWaveOK, busy, msgRaman;

        int checkRcv, polyDeviceStatus, waveCnt,
            commandPosF1, commandPosF2, commandPosF3, commandPosF1W, commandPosF2W, commandPosF4W,
            scanWaveRaman, scanCenterRaman, scanBandwidthRaman, scanRepeatCnt,
            targetPosF1, targetPosF2, targetPosF3, targetPosF1W, targetPosF2W, targetPosF4W,
            posF1, posF2, posF3, posF1W, posF2W, posF4W,
            currentRamanCenter, currentRamanBandwidth, currentRamanStartPoint, currentRamanEndPoint, currentRamanBandwidthLeft, currentRamanBandwidthRight,
            commandCenterRaman, commandBandwidth;

        double scanStart, scanEnd, scanFwhm, scanStep, scanDelay,
            currentCWL, currentFWHM, currentSW, currentLW,
            targetCWL, targetFWHM, targetSW, targetLW,
            calibrated_Center_Raman_shift, possible_bandwidth, commandWavelength, commandFWHM;

        string strRcvData = string.Empty, ElapsedTime = string.Empty, codeName = string.Empty, iniModelName = string.Empty, iniSerialNumber = string.Empty, iniWaveRange = string.Empty,
            pppPath = string.Empty, modelName = string.Empty, firmwareVersion = string.Empty, serialNumber = string.Empty, waveRange = string.Empty;

        char[] delimiterChars = new char[3] { ' ', ',', '\r' };
        double[] wavelist = null;
        string[] waveData = null;

        Stopwatch stopwatch;
        SerialPort sPort = new SerialPort();
        WATCrypt m_crypt = new WATCrypt("JTlabsys");

        public Poly()
        {
            //IL_0001: Unknown result type (might be due to invalid IL or missing references)
            //IL_000b: Expected O, but got Unknown
            this.stopwatch = new Stopwatch();

            this.modelName = "";
            this.serialNumber = "";
            this.waveRange = "";
            this.bSetWaveOK = false;
            this.busy = false;
            this.currentRamanCenter = 0;
            this.currentRamanBandwidth = 0;
            this.currentRamanStartPoint = 0;
            this.currentRamanEndPoint = 0;
            this.currentRamanBandwidthLeft = 0;
            this.currentRamanBandwidthRight = 0;
        }

        public int Disconnect()
        {
            return ClosePort();
        }

        public async Task<int> PolyConnect(string path)
        {
            int num = 0;
            for (int i = 0; i < 10; i++)
            {
                num = await Connection(path);
                if (num == 0)
                {
                    break;
                }

                await Task.Delay(5000);
            }

            return num;
        }

        public async Task<int> GetDeviceStatus()
        {
            if (this.sPort.IsOpen)
            {
                int num = await WriteCommand("STA ?", COMMAND_TIME_OUT);
                if (num == 0)
                {
                    return this.polyDeviceStatus switch
                    {
                        0 => 4,
                        2 => 5,
                        3 => 6,
                        10 => -12,
                        _ => -12,
                    };
                }

                return num;
            }

            return -9;
        }

        public bool GetDeviceEnabled()
        {
            if (this.polyEnabled)
            {
                return this.sPort.IsOpen;
            }

            return false;
        }

        public string GetComPortNumber()
        {
            if (this.polyEnabled && this.sPort.IsOpen)
            {
                return this.sPort.PortName;
            }

            return "";
        }

        public int GetInforData(ref string model, ref string serial, ref string range)
        {
            if (!this.sPort.IsOpen)
            {
                return -9;
            }

            int num = CheckEnable();
            if (num == 0)
            {
                model = this.modelName;
                serial = this.serialNumber;
                range = this.waveRange;
                try
                {
                    this.StartPoint_nm = Convert.ToDouble(range.Split(new char[1] { '-' })[0], CultureInfo.InvariantCulture);
                    this.EndPoint_nm = Convert.ToDouble(range.Split(new char[1] { '-' })[1], CultureInfo.InvariantCulture);
                    return 0;
                }
                catch
                {
                    return -8;
                }
            }

            return num;
        }

        public async Task<int> SetWavelength(string CW, string FWHM)
        {
            if (!this.sPort.IsOpen)
            {
                return -9;
            }

            if (this.busy)
            {
                return 5;
            }

            this.msgRaman = false;

            double.TryParse(CW, CultureInfo.InvariantCulture, out this.targetCWL);
            double.TryParse(FWHM, CultureInfo.InvariantCulture, out this.targetFWHM);

            int num = await SetWave(this.targetCWL, this.targetFWHM);
            if (num == 0)
            {
                this.EventPolyUiUpdate?.Invoke(3);
            }
            else
            {
                this.EventPolyUiUpdate?.Invoke(num);
                this.bScanWaveEnable = false;
            }

            return num;
        }

        public async Task<int> SetWavelength(double CW, double FWHM)
        {
            if (!this.sPort.IsOpen)
            {
                return -9;
            }

            if (this.busy)
            {
                return 5;
            }

            this.msgRaman = false;
            this.targetCWL = CW;
            this.targetFWHM = FWHM;

            int num = await SetWave(this.targetCWL, this.targetFWHM);
            if (num == 0)
            {
                this.EventPolyUiUpdate?.Invoke(3);
            }
            else
            {
                this.EventPolyUiUpdate?.Invoke(num);
                this.bScanWaveEnable = false;
            }

            return num;
        }

        public int GetCurrentWavelength(ref string sw, ref string cw, ref string lw, ref string fwhm)
        {
            if (!this.sPort.IsOpen)
            {
                return -9;
            }

            if (this.bSetWaveOK)
            {
                cw = this.currentCWL.ToString("0.0");
                fwhm = this.currentFWHM.ToString("0");
                sw = (this.currentCWL - this.currentFWHM / 2.0).ToString("0.0");
                lw = (this.currentCWL + this.currentFWHM / 2.0).ToString("0.0");
                return 0;
            }

            return -22;
        }

        public async Task<int> GoBlankPosition()
        {
            if (!this.sPort.IsOpen)
            {
                return -9;
            }

            if (this.busy)
            {
                return 5;
            }

            this.busy = true;
            return await GoPosAll(0, 0, 0, 9, 9, 4);
        }

        public int ScanWavelength(double start, double end, int fwhm, double step, double delay, int repeat)
        {
            this.scanRepeatCnt = repeat;
            return ScanWavelength(start, end, fwhm, step, delay);
        }

        public int ScanWavelength(double start, double end, int fwhm, double step, double delay)
        {
            if (!this.sPort.IsOpen)
            {
                return -9;
            }

            if (this.ScanRunning)
            {
                return 5;
            }

            int num = CheckEnable();
            if (num == 0)
            {
                if (this.busy)
                {
                    return 5;
                }

                this.scanStart = start;
                this.scanEnd = end;
                this.scanFwhm = fwhm;
                this.scanStep = step;
                this.scanDelay = delay * 1000.0;
                num = Findwave_TaskScan();
                if (num == 0)
                {
                    this.bScanWaveEnable = true;
                    Task.Run(TaskScanWave);
                    return 0;
                }

                return num;
            }

            return num;
        }

        public int ScanWavelengthRaman(int wavelength, int start, int end, int bandwidth, int step, double delay, int repeat)
        {
            this.scanRepeatCnt = repeat;
            return ScanWavelengthRaman(wavelength, start, end, bandwidth, step, delay);
        }

        public int ScanWavelengthRaman(int wavelength, int start, int end, int bandwidth, int step, double delay)
        {
            if (!this.sPort.IsOpen)
            {
                return -9;
            }

            if (this.ScanRunning)
            {
                return 5;
            }

            int num = CheckEnable();
            if (num == 0)
            {
                if (this.busy)
                {
                    return 5;
                }

                this.scanWaveRaman = wavelength;
                this.scanBandwidthRaman = bandwidth;
                this.scanStart = start;
                this.scanEnd = end;
                this.scanStep = step;
                this.scanDelay = delay * 1000.0;

                if (this.scanEnd > this.scanStart)
                {
                    this.waveCnt = (int)((this.scanEnd - this.scanStart) / this.scanStep);
                }
                else
                {
                    this.waveCnt = (int)((this.scanStart - this.scanEnd) / this.scanStep);
                }

                this.bScanWaveEnable = true;
                Task.Run(TaskScanRaman);
                return 0;
            }

            return num;
        }

        public async Task<int> DeviceReset()
        {
            int num = CheckEnable();
            if (num == 0)
            {
                this.bcheckOrg = false;
                this.busy = true;
                this.sPort.Write("RST*\r");
                await Task.Delay(1000);
                return 4;
            }

            return num;
        }


        public async Task<(int, string)> GetDeviceList()
        {
            string[] portNames = SerialPort.GetPortNames();
            int num = portNames.Length;
            string deviceList = "";
            if (num == 0)
            {
                return (0, deviceList);
            }

            try
            {
                for (int i = 0; i < num; i++)
                {
                    if (PortOpen(portNames[i]) != 0)
                    {
                        continue;
                    }

                    if (await GetVersion() == 0)
                    {
                        this.firmwareVersion = this.codeName.Split(this.delimiterChars)[1];
                        for (int j = 0; j < 3; j++)
                        {
                            if (await GetDeviceInfo(j) != 0)
                            {
                                ClosePort();
                                return (-14, deviceList);
                            }
                        }

                        deviceList += portNames[i];
                        deviceList += ",";
                        deviceList += this.modelName;
                        deviceList += ",";
                        deviceList += this.serialNumber;
                        deviceList += ",";
                        deviceList += this.waveRange;
                        deviceList += ":";
                    }

                    ClosePort();
                }

                return (0, deviceList);
            }
            catch
            {
                return (-3, deviceList);
            }
        }

        public int GetCurrentRaman(ref int CenterRaman, ref int BandwidthRaman, ref int StartPoint, ref int EndPoint, ref int BandwidthLeft, ref int BandwidtRight)
        {
            CenterRaman = this.currentRamanCenter;
            BandwidthRaman = this.currentRamanBandwidth;
            StartPoint = this.currentRamanStartPoint;
            EndPoint = this.currentRamanEndPoint;
            BandwidthLeft = this.currentRamanBandwidthLeft;
            BandwidtRight = this.currentRamanBandwidthRight;
            return 0;
        }

        public async Task<int> SetRamanStokes(double iExcWavelength, int iCenterRaman, int iBandwidthRaman)
        {
            if (!this.sPort.IsOpen)
            {
                return -9;
            }

            if (this.busy)
            {
                return 5;
            }

            int num = Wave2RamanStokes(iExcWavelength, iCenterRaman, iBandwidthRaman);

            if (num == 0)
            {
                this.msgRaman = true;
                this.commandWavelength = iExcWavelength;
                this.commandCenterRaman = iCenterRaman;
                this.commandBandwidth = iBandwidthRaman;
                this.targetCWL = this.calibrated_Center_Raman_shift;
                this.targetFWHM = this.possible_bandwidth;

                num = await SetWave(this.targetCWL, this.targetFWHM);
                if (num == 0)
                {
                    this.EventPolyUiUpdate?.Invoke(3);
                }
                else
                {
                    this.EventPolyUiUpdate?.Invoke(num);
                    this.bScanWaveEnable = false;
                }
            }

            return num;
        }

        public async Task<int> SetAntiRamanStokes(double iExcWavelength, int iCenterRaman, int iBandwidthRaman)
        {
            if (!this.sPort.IsOpen)
            {
                return -9;
            }

            if (this.busy)
            {
                return 5;
            }

            int num = Wave2AntiRamanStokes(iExcWavelength, iCenterRaman, iBandwidthRaman);
            if (num == 0)
            {
                this.msgRaman = true;
                this.commandWavelength = iExcWavelength;
                this.commandCenterRaman = iCenterRaman;
                this.commandBandwidth = iBandwidthRaman;
                this.targetCWL = this.calibrated_Center_Raman_shift;
                this.targetFWHM = this.possible_bandwidth;
                num = await SetWave(this.targetCWL, this.targetFWHM);
                if (num == 0)
                {
                    this.EventPolyUiUpdate?.Invoke(3);
                }
                else
                {
                    this.EventPolyUiUpdate?.Invoke(num);
                    this.bScanWaveEnable = false;
                }
            }

            return num;
        }

        public string GetStringMsg(int code)
        {
            string result = "";
            switch (code)
            {
                case 0:
                    result = "No error.";
                    this.busy = false;
                    break;
                case 1:
                    result = "Device searching ...";
                    this.busy = true;
                    break;
                case 2:
                    result = "Device connected.";
                    this.busy = false;
                    break;
                case 3:
                    result = ((!this.msgRaman) ? (result = string.Format("Successfully changed wavelength and FWHM. CWL:{0} , FWHM:{1}", this.commandWavelength.ToString("0.0"), this.commandFWHM.ToString("0"))) : (result = string.Format("Successfully changed wavelength. Wavelength:{0},Center:{1},Bandwidth:{2}", this.commandWavelength.ToString("0.0"), this.commandCenterRaman.ToString("0"), this.commandBandwidth.ToString("0"))));
                    this.busy = false;
                    break;
                case 4:
                    result = "Device initializing.";
                    this.busy = true;
                    break;
                case 5:
                    result = "Device busy.";
                    break;
                case 6:
                    result = "Device ready.";
                    this.busy = false;
                    break;
                case 10:
                    result = "Device closed.";
                    this.busy = false;
                    break;
                case -1:
                    result = "ERRDV01 : Device not found. Please connect FWS-";
                    break;
                case -2:
                    result = "ERRDV02 : File not found.";
                    break;
                case -3:
                    result = "ERRDV03 : Device ini file error.";
                    break;
                case -4:
                    result = "ERRDV04: Device not ready.";
                    break;
                case -5:
                    result = "ERRDV05: Error device.";
                    break;
                case -6:
                    result = $"ERRDV06 : Device error. Please check user's manual. (Model No.{this.iniModelName})";
                    break;
                case -7:
                    result = $"ERRDV07 : Device error. Please check user's manual. (Serial No.{this.iniSerialNumber})";
                    break;
                case -8:
                    result = $"ERRDV08 : Device error. Please check user's manual. (Wave Range.{this.iniWaveRange})";
                    break;
                case -9:
                    result = "ERRDV09 : Device not connected.";
                    break;
                case -11:
                    result = "ERRDV10 : Connection error.";
                    break;
                case -12:
                    result = "ERRDV02 : Communication error. Connection lost.";
                    break;
                case -21:
                    result = ((!this.msgRaman) ? string.Format("ERRDV21 : Out of range. CWL:{0} , FWHM:{1}", this.commandWavelength.ToString("0.0"), this.commandFWHM.ToString("0")) : string.Format("ERRDV21 : Out of range. Wavelength:{0},Center:{1},Bandwidth:{2}", this.commandWavelength.ToString("0.0"), this.commandCenterRaman.ToString("0"), this.commandBandwidth.ToString("0")));
                    break;
                case -22:
                    result = ((!this.msgRaman) ? "ERRDV22 : Failed to change wavelength and FWHM." : "ERRDV22 : Failed to change wavelength.");
                    break;
                default:
                    result = "Not found meassage.";
                    break;
                case -14:
                case -13:
                    break;
            }

            return result;
        }

        int CheckEnable()
        {
            if (!this.polyEnabled)
            {
                return -4;
            }

            if (!this.sPort.IsOpen)
            {
                return -9;
            }

            return 0;
        }

        async Task TaskScanRaman()
        {
            for (this.ScanRunning = true; this.scanRepeatCnt >= 0; this.scanRepeatCnt--)
            {
                for (int i = 0; i <= this.waveCnt; i++)
                {
                    if (!this.bScanWaveEnable)
                    {
                        break;
                    }

                    if (this.scanEnd > this.scanStart)
                    {
                        this.scanCenterRaman = (int)Math.Round(this.scanStart + (double)i * this.scanStep, 0);
                    }
                    else
                    {
                        this.scanCenterRaman = (int)Math.Round(this.scanStart - (double)i * this.scanStep, 0);
                    }

                    await SetRamanStokes(this.scanWaveRaman, this.scanCenterRaman, this.scanBandwidthRaman);

                    await Task.Delay((int)this.scanDelay);
                }

                if (!this.bScanWaveEnable)
                {
                    break;
                }
            }

            this.scanRepeatCnt = 0;
            this.ScanRunning = false;
        }

        async Task TaskScanWave()
        {
            this.ScanRunning = true;
            double num = 0.0;
            while (this.scanRepeatCnt >= 0)
            {
                for (int i = 0; i <= this.waveCnt; i++)
                {
                    if (!this.bScanWaveEnable)
                    {
                        break;
                    }

                    if (this.wavelist != null)
                    {
                        num = this.wavelist[i];
                        await SetWavelength(num, this.scanFwhm);
                    }

                    await Task.Delay((int)this.scanDelay);
                }

                if (!this.bScanWaveEnable)
                {
                    break;
                }

                this.scanRepeatCnt--;
            }

            this.scanRepeatCnt = 0;
            this.ScanRunning = false;
        }


        async Task<int> Connection(string path)
        {
            if (!File.Exists(path))
            {
                return -2;
            }

            this.pppPath = path;
            try
            {
                ReadDataFile(this.pppPath);
                string[] portNames = SerialPort.GetPortNames();
                foreach (string portName in portNames)
                {
                    if (PortOpen(portName) != 0)
                    {
                        continue;
                    }

                    this.codeName = "";
                    if (await GetVersion() == 0 && (this.codeName.StartsWith("JTlab_POLY") || this.codeName.StartsWith("JTlab_MONO")))
                    {
                        this.polyEnabled = true;
                        this.firmwareVersion = this.codeName.Split(delimiterChars)[1];
                        for (int j = 0; j < 3; j++)
                        {
                            if (await GetDeviceInfo(j) != 0)
                            {
                                ClosePort();
                                return -14;
                            }
                        }

                        if (!this.modelName.StartsWith(this.iniModelName))
                        {
                            ClosePort();
                            return -6;
                        }

                        if (!this.serialNumber.StartsWith(this.iniSerialNumber))
                        {
                            ClosePort();
                            return -7;
                        }

                        if (!this.waveRange.StartsWith(this.iniWaveRange))
                        {
                            ClosePort();
                            return -8;
                        }

                        return 0;
                    }

                    ClosePort();
                }
            }
            catch
            {
                return -3;
            }

            return -1;
        }

        double convertText2Double(string input)
        {
            string text = input.Trim().Replace(" ", "").Replace(",", ".");
            string[] source = text.Split(new char[1] { '.' });
            if (source.Count() > 1)
            {
                text = string.Join("", source.Take(source.Count() - 1).ToArray());
                text = $"{text}.{source.Last()}";
            }

            return double.Parse(text, CultureInfo.InvariantCulture);
        }

        int Wave2RamanStokes(double iExcWavelength, int iCenterRaman, int iBandwidthRaman)
        {
            try
            {
                this.calibrated_Center_Raman_shift = 0.0;
                this.possible_bandwidth = 0.0;
                double num = Math.Round(10000000.0 / iExcWavelength, 5);
                double num2 = 10000000.0 / (num - (double)iCenterRaman);
                double num3 = Math.Round(num2, 1);
                this.calibrated_Center_Raman_shift = 0.0;

                if (num3 * 10.0 % 2.0 == 0.0)
                {
                    this.calibrated_Center_Raman_shift = num3;
                }
                else if (num3 * 10.0 % 2.0 == 1.0)
                {
                    if (num2 - num3 > 0.0)
                    {
                        this.calibrated_Center_Raman_shift = num3 + 0.1;
                    }
                    else
                    {
                        this.calibrated_Center_Raman_shift = num3 - 0.1;
                    }
                }

                this.calibrated_Center_Raman_shift = Math.Round(this.calibrated_Center_Raman_shift, 1);
                double num4 = Math.Round(10000000.0 / (10000000.0 / this.calibrated_Center_Raman_shift + (double)(iBandwidthRaman / 2)), 1);
                double num5 = Math.Round(10000000.0 / (10000000.0 / this.calibrated_Center_Raman_shift - (double)(iBandwidthRaman / 2)), 1);
                this.possible_bandwidth = Math.Round(num5 - num4, 0);

                double num6 = this.calibrated_Center_Raman_shift - this.possible_bandwidth / 2.0;
                double num7 = this.calibrated_Center_Raman_shift + this.possible_bandwidth / 2.0;

                this.currentRamanCenter = (int)Math.Round(num - 10000000.0 / this.calibrated_Center_Raman_shift, 0);
                this.currentRamanBandwidth = (int)Math.Round(10000000.0 / num6 - 10000000.0 / num7, 0);
                this.currentRamanStartPoint = ((iExcWavelength < this.StartPoint_nm) ? ((int)Math.Round(num - 10000000.0 / this.StartPoint_nm, 0)) : 0);
                this.currentRamanEndPoint = (int)Math.Round(num - 10000000.0 / this.EndPoint_nm, 0);
                this.currentRamanBandwidthLeft = (int)Math.Round(num - 10000000.0 / num6, 0);
                this.currentRamanBandwidthRight = (int)Math.Round(num - 10000000.0 / num7, 0);
                return 0;
            }
            catch
            {
            }

            return -10;
        }

        int Wave2AntiRamanStokes(double iExcWavelength, int iCenterRaman, int iBandwidthRaman)
        {
            try
            {
                this.calibrated_Center_Raman_shift = 0.0;
                this.possible_bandwidth = 0.0;
                double num = Math.Round(10000000.0 / iExcWavelength, 5);
                double num2 = 10000000.0 / (num + (double)iCenterRaman);
                double num3 = Math.Round(num2, 1);
                this.calibrated_Center_Raman_shift = 0.0;
                if (num3 * 10.0 % 2.0 == 0.0)
                {
                    this.calibrated_Center_Raman_shift = num3;
                }
                else if (num3 * 10.0 % 2.0 == 1.0)
                {
                    if (num2 - num3 > 0.0)
                    {
                        this.calibrated_Center_Raman_shift = num3 + 0.1;
                    }
                    else
                    {
                        this.calibrated_Center_Raman_shift = num3 - 0.1;
                    }
                }

                this.calibrated_Center_Raman_shift = Math.Round(this.calibrated_Center_Raman_shift, 1);
                double num4 = Math.Round(10000000.0 / (10000000.0 / this.calibrated_Center_Raman_shift + (double)(iBandwidthRaman / 2)), 1);
                double num5 = Math.Round(10000000.0 / (10000000.0 / this.calibrated_Center_Raman_shift - (double)(iBandwidthRaman / 2)), 1);
                this.possible_bandwidth = Math.Round(num5 - num4, 0);
                double num6 = this.calibrated_Center_Raman_shift - this.possible_bandwidth / 2.0;
                double num7 = this.calibrated_Center_Raman_shift + this.possible_bandwidth / 2.0;
                this.currentRamanCenter = (int)Math.Round(10000000.0 / this.calibrated_Center_Raman_shift - num, 0);
                this.currentRamanBandwidth = (int)Math.Round(10000000.0 / num6 - 10000000.0 / num7, 0);
                this.currentRamanStartPoint = ((iExcWavelength > this.EndPoint_nm) ? ((int)Math.Round(10000000.0 / this.EndPoint_nm - num, 0)) : 0);
                this.currentRamanEndPoint = (int)Math.Round(10000000.0 / this.StartPoint_nm - num, 0);
                this.currentRamanBandwidthLeft = (int)Math.Round(10000000.0 / num6 - num, 0);
                this.currentRamanBandwidthRight = (int)Math.Round(10000000.0 / num7 - num, 0);
                return 0;
            }
            catch
            {
            }

            return -10;
        }

        int Findwave_TaskScan()
        {
            int num = 0;
            if (this.scanEnd > this.scanStart)
            {
                this.waveCnt = (int)((this.scanEnd - this.scanStart) / this.scanStep);
            }
            else
            {
                this.waveCnt = (int)((this.scanStart - this.scanEnd) / this.scanStep);
            }

            this.wavelist = new double[this.waveCnt + 1];
            for (num = 0; num <= this.waveCnt; num++)
            {
                double num2 = ((!(this.scanEnd > this.scanStart)) ? Math.Round(this.scanStart - (double)num * this.scanStep, 1) : Math.Round(this.scanStart + (double)num * this.scanStep, 1));
                int num3 = FindWave(num2, this.scanFwhm);
                this.commandWavelength = num2;
                this.commandFWHM = this.scanFwhm;
                if (num3 == 0)
                {
                    this.wavelist[num] = num2;
                    continue;
                }

                this.bScanWaveEnable = false;
                return num3;
            }

            return 0;
        }

        int FindWave(double cwl, double fwhm)
        {
            try
            {
                if (this.waveData != null)
                {
                    for (int i = 1; i < this.waveData.Length; i++)
                    {
                        if (Convert.ToDouble(this.waveData[i].Split(new char[1] { ',' })[0], CultureInfo.InvariantCulture) != cwl)
                        {
                            continue;
                        }

                        string[] array = this.waveData[i].Split(new char[1] { ',' });
                        foreach (string text in array)
                        {
                            if (Convert.ToDouble(text.Split(new char[1] { ':' })[0]) != fwhm)
                            {
                                continue;
                            }

                            try
                            {
                                this.targetPosF1 = Convert.ToInt16(text.Split(new char[1] { ':' })[1]);
                                this.targetPosF2 = Convert.ToInt16(text.Split(new char[1] { ':' })[2]);
                                this.targetPosF3 = Convert.ToInt16(text.Split(new char[1] { ':' })[3]);
                                if (this.codeName.StartsWith("JTlab_POLY"))
                                {
                                    this.targetPosF1W = Convert.ToInt16(text.Split(new char[1] { ':' })[4]);
                                    this.targetPosF2W = Convert.ToInt16(text.Split(new char[1] { ':' })[5]);
                                    this.targetPosF4W = Convert.ToInt16(text.Split(new char[1] { ':' })[6]);
                                }
                                else
                                {
                                    this.targetPosF1W = 0;
                                    this.targetPosF2W = 0;
                                    this.targetPosF4W = 0;
                                }

                                return 0;
                            }
                            catch
                            {
                                return -21;
                            }
                        }
                    }
                }

                return -21;
            }
            catch
            {
                return -3;
            }
        }

        async Task<int> SetWave(double cwl, double fwhm)
        {
            int num = FindWave(cwl, fwhm);
            this.commandWavelength = cwl;
            this.commandFWHM = fwhm;
            if (num == 0)
            {
                this.bSetWaveOK = true;
                this.currentCWL = cwl;
                this.currentFWHM = fwhm;
                this.currentSW = this.targetSW;
                this.currentLW = this.targetLW;
                num = await GoPosAll(this.targetPosF1, this.targetPosF2, this.targetPosF3, this.targetPosF1W, this.targetPosF2W, this.targetPosF4W);
            }

            return num;
        }

        int ClosePort()
        {
            if (this.sPort.IsOpen)
            {
                this.sPort.Close();
                ((Component)(object)this.sPort).Dispose();
            }

            return 10;
        }

        int PortOpen(string portName)
        {
            //IL_007b: Unknown result type (might be due to invalid IL or missing references)
            //IL_0085: Expected O, but got Unknown
            try
            {
                this.checkRcv = 0;
                this.bcheckOrg = false;
                this.sPort.PortName = portName;
                this.sPort.BaudRate = 38400;
                this.sPort.DataBits = 8;
                this.sPort.Parity = (Parity)0;
                this.sPort.StopBits = (StopBits)1;
                this.sPort.ReadTimeout = 500;
                this.sPort.WriteTimeout = 500;
                this.sPort.DataReceived += new SerialDataReceivedEventHandler(sPort_DataReceived);
                this.sPort.Open();
                if (!this.sPort.IsOpen)
                {
                    return -11;
                }

                return 0;
            }
            catch
            {
                return -11;
            }
        }

        int ReadDataFile(string path)
        {
            string[] array = File.ReadAllLines(path);
            if (array.Length != 0)
            {
                this.waveData = new string[array.Length];
                for (int i = 0; i < array.Length; i++)
                {
                    this.waveData[i] = this.m_crypt.Decrypt(array[i]);
                }

                string[] splited = this.waveData[0].Split(this.delimiterChars);

                this.iniModelName = splited[0];
                this.iniSerialNumber = splited[1];
                this.iniWaveRange = splited[2];
                this.polyEnabled = true;
                return 0;
            }

            return -3;
        }

        void sPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            int bytesToRead = this.sPort.BytesToRead;
            if (bytesToRead == 0)
            {
                return;
            }

            byte[] array = new byte[bytesToRead];
            this.sPort.Read(array, 0, bytesToRead);
            for (int i = 0; i < bytesToRead; i++)
            {
                char c = Convert.ToChar(array[i]);
                this.strRcvData += c;
                if (c == '\r')
                {
                    this.stopwatch.Stop();
                    this.ElapsedTime = this.stopwatch.ElapsedMilliseconds.ToString();
                    ProtocolPoly(this.strRcvData);
                    this.strRcvData = "";
                }
            }
        }

        async Task<int> WriteCommand(string data, int tout)
        {
            try
            {
                if (this.sPort.IsOpen)
                {
                    this.busy = true;
                    this.checkRcv = 1;
                    this.sPort.Write(data + "\r");
                    this.stopwatch.Reset();
                    this.stopwatch.Start();
                    for (int i = 0; i < tout; i++)
                    {
                        await Task.Delay(1);
                        if (this.checkRcv == 0)
                        {
                            return 0;
                        }
                    }

                    return -13;
                }

                this.checkRcv = 0;
                return -4;
            }
            catch
            {
                return -14;
            }
        }

        async Task<int> GetVersion()
        {
            try
            {
                if (this.sPort.IsOpen)
                {
                    return await WriteCommand("?", 300);
                }

                return -4;
            }
            catch
            {
                return -14;
            }
        }

        async Task<int> GetDeviceInfo(int num)
        {
            try
            {
                if (this.sPort.IsOpen)
                {
                    return await WriteCommand("INFO " + num, 500);
                }

                return -4;
            }
            catch
            {
                return -14;
            }
        }

        async Task<int> GoPos(int cs, int pos)
        {
            if (this.sPort.IsOpen)
            {
                string data = "GO " + cs + " " + pos;
                return await WriteCommand(data, COMMAND_TIME_OUT);
            }

            return 1;
        }

        async Task<int> GoPos123(int f1, int f2, int f3)
        {
            if (this.sPort.IsOpen)
            {
                this.commandPosF1 = f1;
                this.commandPosF2 = f2;
                this.commandPosF3 = f3;
                string data = "GO 123 " + f1 + "," + f2 + "," + f3;
                return await WriteCommand(data, COMMAND_TIME_OUT);
            }

            return 1;
        }

        async Task<int> GoPos123(string f1, string f2, string f3)
        {
            if (this.sPort.IsOpen)
            {
                this.commandPosF1 = Convert.ToInt16(f1);
                this.commandPosF2 = Convert.ToInt16(f2);
                this.commandPosF3 = Convert.ToInt16(f3);
                string data = "GO 123 " + f1 + "," + f2 + "," + f3;
                return await WriteCommand(data, COMMAND_TIME_OUT);
            }

            return 1;
        }

        async Task<int> GoPosAll(string st)
        {
            if (this.sPort.IsOpen)
            {
                string[] array = st.Split(this.delimiterChars);
                this.commandPosF1 = Convert.ToInt16(array[0]);
                this.commandPosF2 = Convert.ToInt16(array[1]);
                this.commandPosF3 = Convert.ToInt16(array[2]);
                this.commandPosF1W = Convert.ToInt16(array[3]);
                this.commandPosF2W = Convert.ToInt16(array[4]);
                this.commandPosF4W = Convert.ToInt16(array[5]);
                string data = "GO 6 " + st;

                return await WriteCommand(data, COMMAND_TIME_OUT);
            }

            return 1;
        }

        async Task<int> GoPosAll(string f1, string f2, string f3, string f1w, string f2w, string f4w)
        {
            if (this.sPort.IsOpen)
            {
                string data = "GO 6 " + f1 + "," + f2 + "," + f3 + "," + f1w + "," + f2w + "," + f4w;
                return await WriteCommand(data, COMMAND_TIME_OUT);
            }

            return 1;
        }

        async Task<int> GoPosAll(int f1, int f2, int f3, int f1w, int f2w, int f4w)
        {
            int num = CheckEnable();
            if (num == 0)
            {
                string data = "GO 6 " + f1 + "," + f2 + "," + f3 + "," + f1w + "," + f2w + "," + f4w;
                return await WriteCommand(data, COMMAND_TIME_OUT);
            }

            return num;
        }

        async Task<int> Backlash(int en)
        {
            int num = CheckEnable();
            if (num == 0)
            {
                string data = "BKEN " + en;
                return await WriteCommand(data, 300);
            }

            return num;
        }

        int GoHome(string pos)
        {
            int num = CheckEnable();
            if (num == 0)
            {
                string text = "HOME " + pos + "\r";
                sPort.Write(text);
                return 0;
            }

            return num;
        }

        async Task<int> SetWheelPosF1(string pos)
        {
            string data = "SETFW1 " + pos;
            return await WriteCommand(data, COMMAND_TIME_OUT);
        }

        async Task<int> SetWheelPosF2(string pos)
        {
            string data = "SETFW2 " + pos;
            return await WriteCommand(data, COMMAND_TIME_OUT);
        }

        async Task<int> SetWheelPosF4(string pos)
        {
            string data = "SETFW4 " + pos;
            return await WriteCommand(data, COMMAND_TIME_OUT);
        }

        int WriteData(string data)
        {
            if (this.sPort.IsOpen)
            {
                this.sPort.Write(data + "\r");
            }

            return 0;
        }

        void ProtocolPoly(string tmp)
        {
            string[] array = tmp.Split(this.delimiterChars);
            if (array[0].StartsWith("GO") | array[0].StartsWith("POS"))
            {
                switch (Convert.ToInt16(array[1]))
                {
                    case 0:
                        this.posF1 = Convert.ToInt32(array[2]);
                        this.checkRcv--;
                        break;
                    case 1:
                        this.posF2 = Convert.ToInt32(array[2]);
                        this.checkRcv--;
                        break;
                    case 2:
                        this.posF3 = Convert.ToInt32(array[2]);
                        this.checkRcv--;
                        break;
                    case 3:
                        this.posF1W = Convert.ToInt32(array[2]);
                        this.checkRcv--;
                        break;
                    case 4:
                        this.posF2W = Convert.ToInt32(array[2]);
                        this.checkRcv--;
                        break;
                    case 5:
                        this.posF4W = Convert.ToInt32(array[2]);
                        this.checkRcv--;
                        break;
                    case 6:
                        this.posF1 = Convert.ToInt32(array[2]);
                        this.posF2 = Convert.ToInt32(array[3]);
                        this.posF3 = Convert.ToInt32(array[4]);
                        this.posF1W = Convert.ToInt32(array[5]);
                        this.posF2W = Convert.ToInt32(array[6]);
                        this.posF4W = Convert.ToInt32(array[7]);
                        this.checkRcv--;
                        break;
                    case 123:
                        this.posF1 = Convert.ToInt32(array[2]);
                        this.posF2 = Convert.ToInt32(array[3]);
                        this.posF3 = Convert.ToInt32(array[4]);
                        this.checkRcv--;
                        break;
                }

                this.busy = false;
            }
            else if (array[0].StartsWith("JTlab_"))
            {
                this.codeName = tmp;
                this.checkRcv--;
            }
            else if (array[0].StartsWith("INFO"))
            {
                switch (Convert.ToInt16(array[1]))
                {
                    case 0:
                        this.modelName = array[2];
                        break;
                    case 1:
                        this.serialNumber = array[2];
                        break;
                    case 2:
                        this.waveRange = array[2];
                        break;
                }

                this.checkRcv--;
            }
            else if (array[0].StartsWith("STA"))
            {
                this.polyDeviceStatus = Convert.ToInt16(array[1]);
                this.checkRcv--;
            }
            else if (array[0].StartsWith("PARM"))
            {
                this.checkRcv--;
            }
            else if (array[0].StartsWith("OFFSET"))
            {
                this.checkRcv--;
            }
            else if (array[0].StartsWith("G"))
            {
                this.busy = false;
                this.checkRcv--;
            }
            else if (array[0].StartsWith("NG"))
            {
                this.busy = false;
                this.checkRcv--;
            }
            else if (array[0].StartsWith("BKEN"))
            {
                this.checkRcv--;
            }
            else if (array[0].StartsWith("SETFW1"))
            {
                this.posF1W = Convert.ToInt16(array[1]);
                this.busy = false;
                this.checkRcv--;
            }
            else if (array[0].StartsWith("SETFW2"))
            {
                this.posF2W = Convert.ToInt16(array[1]);
                this.busy = false;
                this.checkRcv--;
            }
            else if (array[0].StartsWith("SETFW4"))
            {
                this.posF4W = Convert.ToInt16(array[1]);
                this.busy = false;
                this.checkRcv--;
            }
            else if (array[0].StartsWith("INFO"))
            {
                this.checkRcv--;
            }
            else if (array[0].EndsWith("Origin"))
            {
                this.checkRcv--;
                if (array[1].StartsWith("OK"))
                {
                    this.bcheckOrg = true;
                }
            }
        }
    }
}
