using FwsPoly.SDK;
using FwsPoly.SDK.Enums;

namespace FwsPoly.Winform
{
    public partial class MainForm : Form
    {
        Poly poly;

        public MainForm()
        {
            InitializeComponent();

            lbPath.Text = string.Empty;
            lbModel.Text = string.Empty;
            lbRange.Text = string.Empty;
            lbPortNo.Text = string.Empty;

            btnGO.Enabled = false;

            poly = new Poly();
        }

        private void btnPath_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog ofd = new OpenFileDialog())
            {
                ofd.Filter = @"ism files (*.ism)|*.ism";

                if (ofd.ShowDialog() == DialogResult.OK)
                {
                    lbPath.Text = ofd.FileName;
                }
            }
        }

        private async void btnConnect_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(lbPath.Text))
            {
                SetEnable(value: false);
                UpdateLabel(label: lbStatus, text: "trying to connect..");

                await Connect(filePath: lbPath.Text);

                SetEnable(value: true);
            }
            else
            {
                MessageBox.Show("  ּ");
            }
        }

        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            int msg = poly.Disconnect();
            UpdateLabel(label: lbStatus, text: poly.GetStringMsg(msg));

            btnGO.Enabled = false;
        }

        private async void btnGO_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(tbCWL.Text) && !string.IsNullOrEmpty(tbFWHM.Text))
            {
                int msg = await this.poly.SetWavelength(CW: tbCWL.Text, FWHM: tbFWHM.Text);
                UpdateLabel(label: lbStatus, text: poly.GetStringMsg(msg));
            }
            else
            {
                MessageBox.Show("CWL, FWHM Է ּ");
            }
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        async Task Connect(string filePath)
        {
            int msg = await this.poly.PolyConnect(filePath);
            UpdateLabel(label: lbStatus, text: poly.GetStringMsg(msg));

            if ((PolyMSG)msg == PolyMSG.MSG_NO_ERROR)
            {
                string model = string.Empty, range = string.Empty, portNo = string.Empty;
                msg = this.poly.GetInforData(model: ref model, serial: ref range, range: ref portNo);
                UpdateLabel(label: lbStatus, text: poly.GetStringMsg(msg));

                if ((PolyMSG)msg == PolyMSG.MSG_NO_ERROR)
                {
                    UpdateLabel(label: lbModel, text: model);
                    UpdateLabel(label: lbRange, text: range);
                    UpdateLabel(label: lbPortNo, text: portNo);

                    await Task.Run(CheckDeviceReady);
                }
            }
        }

        async Task CheckDeviceReady()
        {
            bool isReady = false;

            if (!isReady)
            {
                int msg = await this.poly.GetDeviceStatus();
                UpdateLabel(label: lbStatus, text: poly.GetStringMsg(msg));

                if ((PolyMSG)msg == PolyMSG.MSG_DEVICE_READY)
                {
                    UpdateEnable(control: btnGO, value: true);
                    isReady = true;
                }

                await Task.Delay(100);
            }
        }

        void SetEnable(bool value)
        {
            btnPath.Enabled = value;
            btnConnect.Enabled = value;
            btnDisconnect.Enabled = value;
            btnGO.Enabled = value;
            btnExit.Enabled = value;
            tbCWL.Enabled = value;
            tbFWHM.Enabled = value;
        }

        void UpdateEnable(Control control, bool value)
        {
            if (control.InvokeRequired)
            {
                control.BeginInvoke(new Action<Control, bool>(UpdateEnable), control, value);
            }
            else
            {
                control.Enabled = value;
            }
        }

        void UpdateLabel(Label label, string text)
        {
            if (label.InvokeRequired)
            {
                label.BeginInvoke(new Action<Label, string>(UpdateLabel), label, text);
            }
            else
            {
                label.Text = text;
            }
        }

        void UpdateTextbox(TextBox textbox, string text)
        {
            if (textbox.InvokeRequired)
            {
                textbox.BeginInvoke(new Action<TextBox, string>(UpdateTextbox), textbox, text);
            }
            else
            {
                textbox.Text = text;
            }
        }
    }
}